/*
   Compat definitions for qrt kernel modules to cope with different
   kernel versions and changes in the proc api over time

   Author: Kees Cook <keescook@chromium.org>
   Copyright 2011 ChromeOS Authors
   Author: Steve Beattie <steve.beattie@canonical.com>
   Copyright 2020 Canonical, Ltd.
*/

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,25)
# define PROC_CREATE(entry, name, mode, func) \
    entry = proc_create(name, mode, NULL, func)
#else
/* older kernel support... */
# define PROC_CREATE(entry, name, mode, func) { \
    entry = create_proc_entry(name, mode, NULL); \
    if (entry) { \
        entry->proc_fops = func; \
        entry->owner = THIS_MODULE; \
    } \
}
#endif

#if LINUX_VERSION_CODE > KERNEL_VERSION(5,6,0)
# define PROC_OPS proc_ops
# define PROC_OPS_OPEN    proc_open
# define PROC_OPS_READ    proc_read
# define PROC_OPS_LSEEK   proc_lseek
# define PROC_OPS_RELEASE proc_release
#else
# define PROC_OPS file_operations
# define PROC_OPS_OPEN    open
# define PROC_OPS_READ    read
# define PROC_OPS_LSEEK   llseek
# define PROC_OPS_RELEASE release
#endif
